Package: rolog
Type: Package
Title: Query 'SWI'-'Prolog' from R
Version: 0.9.28
Date: 2026-06-24
Authors@R: c(person("Matthias", "Gondan", role=c("aut", "com", "cre"),
  email="Matthias.Gondan-Rochon@uibk.ac.at", comment="University of Innsbruck"),
  person("European Commission", role="fnd",
  comment="Erasmus+ Programme, 2019-1-EE01-KA203-051708"))
Maintainer: Matthias Gondan <Matthias.Gondan-Rochon@uibk.ac.at>
Description: This R package connects to SWI-Prolog, <https://www.swi-prolog.org/>, so that R can send deterministic and non-deterministic queries to prolog (consult, query/submit, once, findall).
License: FreeBSD
Imports: Rcpp (>= 1.0.7), methods, utils
Depends: R (>= 4.2)
URL: https://github.com/mgondan/rolog
BugReports: https://github.com/mgondan/rolog/issues
LinkingTo: Rcpp, rswipl (>= 10.1.5)
RoxygenNote: 7.2.3
Encoding: UTF-8
SystemRequirements: GNU Make
Suggests: rmarkdown, knitr, DiagrammeR, DiagrammeRsvg, rswipl (>=
        10.1.5), htmltools, testthat (>= 3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr, rmarkdown
NeedsCompilation: yes
Packaged: 2026-06-25 09:55:46 UTC; matthias
Author: Matthias Gondan [aut, com, cre] (University of Innsbruck),
  European Commission [fnd] (Erasmus+ Programme,
    2019-1-EE01-KA203-051708)
Repository: CRAN
Date/Publication: 2026-06-25 11:50:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-06-25 13:39:36 UTC; unix
Archs: rolog.so.dSYM
