# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

.query <- function(query, options, env) {
    .Call('_rolog_query_', PACKAGE = 'rolog', query, options, env)
}

.clear <- function() {
    .Call('_rolog_clear_', PACKAGE = 'rolog')
}

.submit <- function() {
    .Call('_rolog_submit_', PACKAGE = 'rolog')
}

.once <- function(query, options, env) {
    .Call('_rolog_once_', PACKAGE = 'rolog', query, options, env)
}

.findall <- function(query, options, env) {
    .Call('_rolog_findall_', PACKAGE = 'rolog', query, options, env)
}

.consult <- function(files) {
    .Call('_rolog_consult_', PACKAGE = 'rolog', files)
}

.portray <- function(query, options) {
    .Call('_rolog_portray_', PACKAGE = 'rolog', query, options)
}

.call <- function(query) {
    .Call('_rolog_call_', PACKAGE = 'rolog', query)
}

.init <- function(argv0) {
    .Call('_rolog_init_', PACKAGE = 'rolog', argv0)
}

.done <- function() {
    .Call('_rolog_done_', PACKAGE = 'rolog')
}

