% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rolog.R
\name{rolog_done}
\alias{rolog_done}
\title{Clean up when detaching the library}
\usage{
rolog_done()
}
\value{
`TRUE` on success
}
\description{
Clean up when detaching the library
}
